<?php
/*
Plugin Name: Simple Trackback Validation
Plugin URI: http://sw-guide.de/wordpress/simple-trackback-validation-plugin/
Description: Eliminates spam trackbacks by retrieving the web page located at the URL used in the trackback and checking if the page contains a link to your blog: if not, the trackback needs to be moderated and is not being published.
Version: 1.2
Author: Michael Woehrer
Author URI: http://sw-guide.de
 	    ____________________________________________________
       |                                                    |
       |        Simple Trackback Validation Plugin          |
       |                © Michael Woehrer                   |
       |____________________________________________________|

	© Copyright 2006  Michael Woehrer (michael dot woehrer at gmail dot com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	----------------------------------------------------------------------------

	ACKNOWLEDGEMENTS:
	- Thanks to Dan Sandler and Andy Thomas at
	  http://seclab.cs.rice.edu/proj/trackback/trackback-validator-plugin/ for
	  their plugin "Trackback Validator". The main idea comes from their plugin.
	  Also, they pointed me to the Snoopy class. 
	- Thanks to Monte Ohrt for his great Snoopy class 
	  (http://snoopy.sourceforge.net/), without that class this plugin would
	  not work.

	----------------------------------------------------------------------------

	INSTALLATION, USAGE:
	Visit the plugin's homepage.

	----------------------------------------------------------------------------
*/


/*******************************************************************************
 * Purpose:		Get plugin options
*******************************************************************************/
$stbv_opt = get_option('plugin_simple_tb_validation');

if ( !is_array($stbv_opt) ) {
	// Options do not exist or have not yet been loaded so we define standard options
	$stbv_opt = array(
		'stbv_action' => 'moderation',
		'stbv_accuracy' => 'strict',
		'stbv_blogurls' => get_bloginfo('url'),
	);	
}

/*******************************************************************************
 * Purpose:		Apply the plugin to Wordpress.
 * 				'trackback_post' executes when a comment is added via 
 * 				trackback.php. It receives the comment ID as a parameter.
*******************************************************************************/
add_action('trackback_post', 'stbv_main', 1, 1);


/*******************************************************************************
 * Function:	stbv_main
 * Purpose:		Main function that retrieves all URLs from the web page of the
 * 				trackback author URL and checks if it really contains a link
 * 				to the blog. If not, the trackback is marked as spam.  
 * Input:		Comment ID
 * Output:		Comment ID
*******************************************************************************/
function stbv_main($stbvCommentID) {

	global $stbv_opt, $wpdb, $table_prefix;

	/////////////////////////////////////////
	// Get Author URL and Comment's Post ID of comment in question
	/////////////////////////////////////////
	$stbvCommentInfo = $wpdb->get_row("
		SELECT comment_author_url, comment_post_ID 
		FROM $wpdb->comments 
		WHERE comment_ID = '$stbvCommentID'
		", ARRAY_A);
	$stbvCommentAuthorURL 	= $stbvCommentInfo['comment_author_url'];
	$stbvPermalink 			= get_permalink($stbvCommentInfo['comment_post_ID']);
	$stbvPermalink 			= preg_replace('/\/$/', '', $stbvPermalink); // Remove trailing slash
	
	/////////////////////////////////////////
	// Get Plugin options
	/////////////////////////////////////////
	if ($stbv_opt['stbv_accuracy'] == 'open') {
		if ( is_string($stbv_opt['stbv_blogurls']) ) {
			if (strlen($stbv_opt['stbv_blogurls']) > 9) {
				$stbv_blogurlsArray = explode(' ', $stbv_opt['stbv_blogurls']);
			}
		}
	}
	
		
	/////////////////////////////////////////
	// 'Is Spam' flag is FALSE by default. Below we check several things
	// and this flag will become true as soon as we have any doubts.
	/////////////////////////////////////////
	$stbvIsSpam = false;

	/////////////////////////////////////////
	// If Author's URL is not correct, so might be spam.
	/////////////////////////////////////////
	if (!$stbvIsSpam && substr($stbvCommentAuthorURL, 0, 4) != 'http') {
		$stbvIsSpam = true;
	}

	/////////////////////////////////////////
	// Loading snoopy and create snoopy object. In case of failure it is being considered as spam, just in case.
	/////////////////////////////////////////
	if (!$stbvIsSpam && !stbv_loadSnoopy() ) {
		// Loading snoopy failed
		$stbvIsSpam = true;
	} else { 
		// Create new Snoopy object
		$stbvSnoopy = new Snoopy;
	}

	/////////////////////////////////////////
	// Fetch all URLs of the author's web page
	/////////////////////////////////////////
	if (!$stbvIsSpam && ! @$stbvSnoopy->fetchlinks($stbvCommentAuthorURL) ) {
			// Snoopy couldn't find something, Snoopy error occurred, or something else...
			$stbvIsSpam = true;	
	} else {
		$stbvAuthorUrlArray = $stbvSnoopy->results;
	}


	/////////////////////////////////////////
	// Check if URL array contains link to website
	/////////////////////////////////////////
	if (!$stbvIsSpam && is_array($stbvAuthorUrlArray) ) {
		$loopSuccess = false;
		
		foreach ($stbvAuthorUrlArray as $loopUrl) {

			// Remove trailing slash, "/trackback" and "/trackback/"
			$loopUrl = preg_replace('/(\/|\/trackback|\/trackback\/)$/', '', $loopUrl);
			

			if ( ($stbv_opt['stbv_accuracy'] == 'open') && (is_array($stbv_blogurlsArray)) ) {
				// We have more than one URL to be checked
				$loopInnerSuccess = false;
				
				foreach ($stbv_blogurlsArray as $loopOptionsURL) {
					// Check if the first chars of the URL of remote page contain URL of the options 		
					if (substr($loopUrl, 0, strlen($loopOptionsURL)) == $loopOptionsURL) {
						$loopInnerSuccess = true; 
						break;
					}
				}
				if ( $loopInnerSuccess ) { 
					$loopSuccess = true;
					break;
				}
			} else {
				// Strict mode or no URLs provided so we check strictly the permalink only!
				if ( $loopUrl == $stbvPermalink ) {
					$loopSuccess = true;
					break;
				}
			}
		}
		if ( !$loopSuccess ) { $stbvIsSpam = true; }
	}

	/////////////////////////////////////////
	// Now we know if we have a trackback spam or not. 
	/////////////////////////////////////////
	if ( !$stbvIsSpam ) {
		// **** No Trackback Spam ***
		return $stbvCommentID; // Exit if we have not found any spam

	} else {
		// **** It is Trackback Spam ***
		// We put trackback in moderation queue, mark as spam or delete right away	
		switch ($stbv_opt['stbv_action']) {
			case 'delete':	$stbvTBaction = 'delete';	break;
			case 'spam':	$stbvTBaction = 'spam';		break;
			default:		$stbvTBaction = 'hold';	
		}
		wp_set_comment_status($stbvCommentID, $stbvTBaction);
	}
	
	
} // function stbv_main()


/*******************************************************************************
 * Function:	stbv_loadSnoopy
 * Purpose:		Load the Snoopy class.
 * Input:		-
 * Output:		TRUE if class is successfully loaded, FALSE otherwise.
*******************************************************************************/
function stbv_loadSnoopy() {
	if ( !class_exists('Snoopy') ) {

		if (@include_once( ABSPATH . WPINC . '/class-snoopy.php' )) {
			return true;
		} else {
			return false;
		}
	} else {
		return true;
	}
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////       ADMIN       /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


/*******************************************************************************
 * Function:	stbv_add_options_to_admin
 * Purpose:		Apply Options to Admin Menu
 * Input:		-
 * Output:		-
*******************************************************************************/
function stbv_add_options_to_admin() {
    if (function_exists('add_options_page')) {
		add_options_page('Simple TB Validation', 'Simple TB Validation', 8, basename(__FILE__), 'stbv_adminOptions');
    }
}


/*******************************************************************************
 * Function:	stbv_adminOptions
 * Purpose:		This will add the new item, 'Simple TB Validation', to the 
 * 				Options menu.
 * Input:		-
 * Output:		-
*******************************************************************************/
function stbv_adminOptions() {

	global $stbv_opt;

	add_option('plugin_simple_tb_validation', $stbv_opt, 'Simple Trackback Validation Plugin Options');

	/* Check form submission and update options if no error occurred */
	if (isset($_POST['submit']) ) {

		// Options array
		$stbv_opt_update = array (
			'stbv_blogurls' => stbv_txtLineBreakToWhiteSpace($_POST['stbv_blogurls']),
			'stbv_action' => $_POST['stbv_action'],
			'stbv_accuracy' => $_POST['stbv_accuracy'],
		);
		update_option('plugin_simple_tb_validation', $stbv_opt_update);

	}

	/* Get options */
	$stbv_opt = get_option('plugin_simple_tb_validation');

	
	?>
	<div class=wrap>
	
		<h2>Simple Trackback Validation Options</h2>

		<? if (stbv_isOldWordpress()) echo '<p style="color: red; font-weight: bold">You are using an outdated Wordpress version which is not supported by this plugin. Get the latest version at <a href="http://wordpress.org/download/">wordpress.org</a>.</p>'; ?>
	
		<form name="form1" method="post" action="<?php echo $_SERVER['PHP_SELF'] . '?page=' . basename(__FILE__); ?>&updated=true">
	
		<fieldset class="options">
			<h3>How to deal with spam trackbacks?</h3>

				<input id="radioa1" type="radio" name="stbv_action" value="moderation" <?php echo ($stbv_opt['stbv_action']=='moderation'?'checked="checked"':'') ?> />
				<label for="radioa1">Place into comment moderation queue</label> <span style="color: grey; font-size: 90%;"> (<a href="<?php bloginfo('url'); ?>/wp-admin/moderation.php">Awaiting Moderation &raquo;</a>)</span>
				<br />
				<input id="radioa2" type="radio" name="stbv_action" value="spam" <?php echo ($stbv_opt['stbv_action']=='spam'?'checked="checked"':'') ?> />
				<label for="radioa2">Mark as spam</label> <span style="color: grey; font-size: 90%;">(a plugin like <a href="http://akismet.com/">Akismet</a> will be necessary to access to these trackbacks)</span>
				<br />
				<input id="radioa3" type="radio" name="stbv_action" value="delete" <?php echo ($stbv_opt['stbv_action']=='delete'?'checked="checked"':'') ?> />
				<label for="radioa3">Delete immediately <span style="color: grey; font-size: 90%;">(Restoring is not possible)</span></label>

			<h3>Strictness</h3>

				<input id="radiob1" type="radio" name="stbv_accuracy" value="strict" <?php echo ($stbv_opt['stbv_accuracy']!='open'?'checked="checked"':'') ?> />
				<label for="radiob1">Strict: A permalink needs to be used</label>
				<p style="margin-left: 35px; color: grey; font-size: 90%;">That means that e.g. a link to your blog's home (<?php bloginfo('url'); ?>) will not be accepted; 
				If the permalink of your post is not available on the trackback's source page, the trackback is considered as spam.   
				</p>
				<input id="radiob2" type="radio" name="stbv_accuracy" value="open" <?php echo ($stbv_opt['stbv_accuracy']=='open'?'checked="checked"':'') ?> />
				<label for="radiob2">Any link beginning with the following URLs is allowed:</label>
				<br />
				<textarea style="margin: 10px 0 0 35px" name="stbv_blogurls" id="stbv_blogurls" cols="100%" rows="2" ><?php echo stbv_txtWhiteSpaceToLineBreak($stbv_opt['stbv_blogurls']); ?></textarea>
				<p style="margin-left: 35px; color: grey; font-size: 90%;">
				Separate multiple URLs with new lines.<br />
				If you enter for example your blog's URL (<span style="color: #073991;"><?php bloginfo('url'); ?></span>), any link that begins with that URL, 
				e.g. "<em><?php bloginfo('url'); ?>/about-me/</em>, will be accepted, even if it is completely different to the actual permalink of the post.			
				</p>

			<h3>Further information</h3>
			<p>Visit the <a href="http://sw-guide.de/wordpress/simple-trackback-validation-plugin/">plugin's homepage</a> for further information or grabbing the latest version of this plugin.

		</fieldset>
	
			<div class="submit">
				<input type="submit" name="submit" value="<?php _e('Update Options') ?> &raquo;" />
			</div>

		</form>
		<p style="margin-top: 30px; text-align: center; font-size: .85em;">&copy; Copyright 2006&nbsp;&nbsp;<a href="http://sw-guide.de">Michael W&ouml;hrer</a></p>
	
	</div> <!-- [wrap] -->
	<?php


} // function mwli_options_subpanel


/*******************************************************************************
 * Function:	stbv_txtLineBreakToWhiteSpace
 * Purpose:		Converts textarea content (separated by line break) to space 
 * 				separated string since we want to store it like this in MySQL
 * Input:		String
 * Output:		String
*******************************************************************************/
function stbv_txtLineBreakToWhiteSpace($input) {

	// Replace multiple whitespaces with only one space
	$input = preg_replace('/\s\s+/', ' ', $input);
	
	// Replace white space with line break
	$input = str_replace(' ', "\n", $input);	
	
	// Replace linebreaks with white space, considering both \n and \r
	$input = preg_replace("/\r|\n/s", ' ', $input);

	// Create result. We create an array and loop thru it but do not consider empty values. 
	$sourceArray = explode(' ', $input);
	$loopcount = 0;
	$result = '';
	foreach ($sourceArray as $loopval) {

		if ($loopval <> '') {

			// Clean URL (it's a Wordpress function)
			$loopval = clean_url($loopval);
			
			// Create separator
			$sep = ''; 
			if ($loopcount >= 1) $sep = ' ';
			
			// result
			$result .= $sep . $loopval;
		
			$loopcount++;				
		}
	}
	return $result;

}


/*******************************************************************************
 * Function:	stbv_txtWhiteSpaceToLineBreak
 * Purpose:		Replace white space with new line for displaying in text area 
 * Input:		String
 * Output:		String
*******************************************************************************/
function stbv_txtWhiteSpaceToLineBreak($input) {

	$output = str_replace(' ', "\n", $input);
	
	return $output;

}

/*******************************************************************************
 * Function:	stbv_isOldWordpress
 * Purpose:		Check if the current Wordpress installation is 1.x. 
 * Input:		-
 * Output:		TRUE if it is Wordpress 1.x, FALSE otherwise.
*******************************************************************************/
function stbv_isOldWordpress() {

	global $wp_version;

	if (preg_match("/^1\./", $wp_version)) { 
		return true;
	} else {
		return false;
	}

}


/*******************************************************************************
 * Purpose:		Apply the admin menu
*******************************************************************************/
add_action('admin_menu', 'stbv_add_options_to_admin');

?>